"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createActions = exports.createAction = exports.getActionName = exports.SUPPORTED_REDUCE_OPTS = exports.SUPPORTED_FETCH_OPTS = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));

var _pipeline = require("../defaults/pipeline");

var _fetch = _interopRequireWildcard(require("../helpers/fetch"));

var _url = require("../helpers/url");

var _util = require("../helpers/util");

var _types = require("../types");

var _transform = require("./transform");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2["default"])(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

var SUPPORTED_FETCH_OPTS = ['url', 'method', 'headers', 'credentials', 'query', 'body', 'signal'];
exports.SUPPORTED_FETCH_OPTS = SUPPORTED_FETCH_OPTS;
var SUPPORTED_REDUCE_OPTS = ['assignResponse', 'gerundName', 'invalidateState', 'isArray', 'isPure', 'mergeResponse', 'params', 'reduce'];
exports.SUPPORTED_REDUCE_OPTS = SUPPORTED_REDUCE_OPTS;

var getActionName = function getActionName(actionId, _ref) {
  var resourceName = _ref.resourceName,
      _ref$resourcePluralNa = _ref.resourcePluralName,
      resourcePluralName = _ref$resourcePluralNa === void 0 ? (0, _util.getPluralName)(resourceName) : _ref$resourcePluralNa,
      _ref$isArray = _ref.isArray,
      isArray = _ref$isArray === void 0 ? false : _ref$isArray,
      alias = _ref.alias;
  return !resourceName ? actionId : "".concat(alias || actionId).concat((0, _util.ucfirst)(isArray ? resourcePluralName : resourceName));
};

exports.getActionName = getActionName;

var createAction = function createAction(actionId, _ref2) {
  var scope = _ref2.scope,
      _ref2$stripTrailingSl = _ref2.stripTrailingSlashes,
      stripTrailingSlashes = _ref2$stripTrailingSl === void 0 ? true : _ref2$stripTrailingSl,
      actionOpts = (0, _objectWithoutProperties2["default"])(_ref2, ["scope", "stripTrailingSlashes"]);
  var type = (0, _types.scopeType)((0, _types.getActionType)(actionId), scope); // Actual action function with two args
  // Context usage changes with resolved method:
  // - GET/DELETE will be used to resolve query params (eg. /users/:id)
  // - POST/PATCH will be used to resolve query params (eg. /users/:id) and as request body

  return function (context) {
    var contextOpts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    return function (dispatch, getState) {
      // Prepare reduce options
      var reduceOpts = _objectSpread(_objectSpread({}, _util.pick.apply(void 0, [actionOpts].concat(SUPPORTED_REDUCE_OPTS))), _util.pick.apply(void 0, [contextOpts].concat(SUPPORTED_REDUCE_OPTS))); // Support pure actions


      if (actionOpts.isPure) {
        dispatch({
          type: type,
          status: 'resolved',
          options: reduceOpts,
          context: context
        });
        return Promise.resolve(null);
      } // First dispatch a pending action


      dispatch({
        type: type,
        status: 'pending',
        options: reduceOpts,
        context: context
      }); // Prepare fetch options

      var fetchOpts = _objectSpread(_objectSpread({}, _util.pick.apply(void 0, [actionOpts].concat(SUPPORTED_FETCH_OPTS))), _util.pick.apply(void 0, [contextOpts].concat(SUPPORTED_FETCH_OPTS))); // Support dynamic fetch options


      var resolvedfetchOpts = Object.keys(fetchOpts).reduce(function (soFar, key) {
        soFar[key] = (0, _util.isFunction)(fetchOpts[key]) ? fetchOpts[key](getState, {
          context: context,
          contextOpts: contextOpts,
          actionId: actionId
        }) : fetchOpts[key];
        return soFar;
      }, {});
      var _resolvedfetchOpts$ur = resolvedfetchOpts.url,
          url = _resolvedfetchOpts$ur === void 0 ? '' : _resolvedfetchOpts$ur,
          eligibleFetchOptions = (0, _objectWithoutProperties2["default"])(resolvedfetchOpts, ["url"]); // Build fetch url and options

      var urlParams = (0, _url.parseUrlParams)(url);
      var finalFetchOpts = (0, _fetch.buildFetchOpts)(context, eligibleFetchOptions);
      var finalFetchUrl = (0, _fetch.buildFetchUrl)(context, {
        url: url,
        method: finalFetchOpts.method,
        params: reduceOpts.params,
        urlParams: urlParams,
        stripTrailingSlashes: stripTrailingSlashes,
        isArray: actionOpts.isArray
      });
      return (0, _fetch["default"])(finalFetchUrl, finalFetchOpts).then(_fetch.serializeResponse).then((0, _transform.applyTransformPipeline)((0, _transform.buildTransformPipeline)(_pipeline.defaultTransformResponsePipeline, actionOpts.transformResponse))).then(function (payload) {
        dispatch({
          type: type,
          status: 'resolved',
          context: context,
          options: reduceOpts,
          payload: payload
        });
        return payload;
      })["catch"](function (err) {
        var payload = err instanceof _fetch.HttpError ? {
          body: err.body,
          headers: err.headers,
          code: err.status,
          // deprecated
          status: err.status,
          ok: false,
          receivedAt: Date.now()
        } : {
          body: err.message,
          headers: {},
          code: 0,
          // deprecated
          status: 0,
          ok: false,
          receivedAt: Date.now()
        }; // beforeError hook

        var nextErr = actionOpts.beforeError ? actionOpts.beforeError.reduce(function (errSoFar, beforeErrorHook) {
          return errSoFar ? beforeErrorHook(errSoFar) : null;
        }, err) : err;
        dispatch({
          type: type,
          status: 'rejected',
          context: context,
          options: reduceOpts,
          payload: payload
        });

        if (nextErr === null) {
          return payload;
        }

        if (!(nextErr instanceof Error)) {
          return nextErr;
        }

        throw nextErr;
      });
    };
  };
};

exports.createAction = createAction;

var createActions = function createActions() {
  var actions = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  var _ref3 = arguments.length > 1 ? arguments[1] : undefined,
      resourceName = _ref3.resourceName,
      _ref3$resourcePluralN = _ref3.resourcePluralName,
      resourcePluralName = _ref3$resourcePluralN === void 0 ? (0, _util.getPluralName)(resourceName) : _ref3$resourcePluralN,
      _ref3$scope = _ref3.scope,
      scope = _ref3$scope === void 0 ? (0, _types.getTypesScope)(resourceName) : _ref3$scope,
      globalOpts = (0, _objectWithoutProperties2["default"])(_ref3, ["resourceName", "resourcePluralName", "scope"]);

  var actionKeys = Object.keys(actions);
  return actionKeys.reduce(function (actionFuncs, actionId) {
    // Add support for relative url override
    var url = actions[actionId].url;

    if (globalOpts.url && url && (0, _util.isString)(url) && url.substr(0, 1) === '.') {
      actions[actionId] = _objectSpread(_objectSpread({}, actions[actionId]), {}, {
        url: "".concat(globalOpts.url).concat(url.substr(1))
      });
    }

    var actionOpts = actions[actionId];
    var actionName = actionOpts.name ? actionOpts.name : getActionName(actionId, {
      resourceName: resourceName,
      resourcePluralName: resourcePluralName,
      isArray: actionOpts.isArray,
      alias: actionOpts.alias
    });
    actionFuncs[actionName] = createAction(actionId, _objectSpread(_objectSpread({
      scope: scope
    }, globalOpts), actionOpts));
    return actionFuncs;
  }, {});
};

exports.createActions = createActions;
//# sourceMappingURL=index.js.map