import { Context, FetchOptions, ContentRange } from '../typings';
export declare class HttpError extends Error {
    statusCode: number;
    status: number;
    body: unknown;
    headers: Record<string, string>;
    constructor(statusCode: number | undefined, { body, message, headers }: {
        body?: unknown;
        message?: string;
        headers?: Response['headers'];
    });
}
declare type BuildFetchUrlOptions = {
    url: string;
    urlParams: Record<string, {
        isQueryParamValue?: boolean;
    }>;
    stripTrailingSlashes?: boolean;
    method?: string;
    params?: Record<string, string>;
    isArray?: boolean;
};
export declare const buildFetchUrl: (context: Context, { url, method, urlParams, params, isArray, stripTrailingSlashes }: BuildFetchUrlOptions) => string;
export declare const buildFetchOpts: (context: Context, { method, headers, credentials, query, body }: FetchOptions) => FetchOptions;
export declare const parseResponseBody: <T = unknown>(res: Response) => Promise<T>;
export interface SerializableResponse<T = unknown> {
    body: T;
    code: Response['status'];
    status: Response['status'];
    headers: Record<string, string>;
    ok: Response['ok'];
    contentRange?: ContentRange | null;
    receivedAt: number;
    [s: string]: unknown;
}
export declare const serializeResponse: <T = unknown>(res: Response) => Promise<SerializableResponse<T>>;
declare const fetch: (url: string, options?: FetchOptions) => Promise<Response>;
export default fetch;
