"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.splitUrlByProtocolAndDomain = exports.replaceQueryStringParamFromUrl = exports.replaceUrlParamFromUrl = exports.parseUrlParams = exports.encodeUriSegment = exports.encodeUriQuery = void 0;
// https://github.com/angular/angular.js/blob/master/src/ngResource/resource.js#L473
var PROTOCOL_AND_DOMAIN_REGEX = /^https?:\/\/[^/]*/;
var NUMBER_REGEX = /^[0-9]+$/;
/**
 * This method is intended for encoding *key* or *value* parts of query component. We need a
 * custom method because encodeURIComponent is too aggressive and encodes stuff that doesn't
 * have to be encoded per http://tools.ietf.org/html/rfc3986
 */

var encodeUriQuery = function encodeUriQuery(val, pctEncodeSpaces) {
  return encodeURIComponent(val).replace(/%40/gi, '@').replace(/%3A/gi, ':').replace(/%24/g, '$').replace(/%2C/gi, ',').replace(/%20/g, pctEncodeSpaces ? '%20' : '+');
};
/**
 * We need our custom method because encodeURIComponent is too aggressive and doesn't follow
 * http://www.ietf.org/rfc/rfc3986.txt with regards to the character set
 * (pchar) allowed in path segments
 */


exports.encodeUriQuery = encodeUriQuery;

var encodeUriSegment = function encodeUriSegment(val) {
  return encodeUriQuery(val, true).replace(/%26/gi, '&').replace(/%3D/gi, '=').replace(/%2B/gi, '+');
};

exports.encodeUriSegment = encodeUriSegment;

var parseUrlParams = function parseUrlParams(url) {
  return url.split(/\W/).reduce(function (urlParams, param) {
    if (!NUMBER_REGEX.test(param) && param && new RegExp("(^|[^\\\\]):".concat(param, "(\\W|$)")).test(url)) {
      urlParams[param] = {
        // eslint-disable-line no-param-reassign
        isQueryParamValue: new RegExp("\\?.*=:".concat(param, "(?:\\W|$)")).test(url)
      };
    }

    return urlParams;
  }, {});
};

exports.parseUrlParams = parseUrlParams;

var replaceUrlParamFromUrl = function replaceUrlParamFromUrl(url, urlParam) {
  var replace = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : '';
  return url.replace(new RegExp("(/?):".concat(urlParam, "(\\W|$)"), 'g'), function (_match, leadingSlashes, tail) {
    return (replace ? leadingSlashes : '') + replace + tail;
  });
};

exports.replaceUrlParamFromUrl = replaceUrlParamFromUrl;

var replaceQueryStringParamFromUrl = function replaceQueryStringParamFromUrl(url, key, value) {
  var re = new RegExp("([?&])".concat(key, "=.*?(&|$)"), 'i');
  var sep = url.indexOf('?') !== -1 ? '&' : '?';
  return url.match(re) ? url.replace(re, "$1".concat(key, "=").concat(value, "$2")) : "".concat(url).concat(sep).concat(key, "=").concat(value);
};

exports.replaceQueryStringParamFromUrl = replaceQueryStringParamFromUrl;

var splitUrlByProtocolAndDomain = function splitUrlByProtocolAndDomain(url) {
  var protocolAndDomain = '';
  var remainderUrl = url.replace(PROTOCOL_AND_DOMAIN_REGEX, function (match) {
    protocolAndDomain = match;
    return '';
  });
  return [protocolAndDomain, remainderUrl];
};

exports.splitUrlByProtocolAndDomain = splitUrlByProtocolAndDomain;
//# sourceMappingURL=url.js.map