"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getActionTypeKey = exports.getActionType = exports.createTypes = exports.createType = exports.getTypesScope = exports.scopeTypes = exports.scopeType = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _util = require("./helpers/util");

var scopeType = function scopeType(type, scope) {
  return scope ? "".concat(scope, "/").concat(type) : type;
};

exports.scopeType = scopeType;

var scopeTypes = function scopeTypes() {
  var types = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var scope = arguments.length > 1 ? arguments[1] : undefined;
  return scope ? (0, _util.mapObject)(types, function (type) {
    return scopeType(type, scope);
  }) : types;
};

exports.scopeTypes = scopeTypes;

var getTypesScope = function getTypesScope(resourceName) {
  return resourceName ? "@@resource/".concat((0, _util.upperSnakeCase)(resourceName)) : '';
};

exports.getTypesScope = getTypesScope;

var getActionTypeKey = function getActionTypeKey(actionId, _ref) {
  var resourceName = _ref.resourceName,
      _ref$resourcePluralNa = _ref.resourcePluralName,
      resourcePluralName = _ref$resourcePluralNa === void 0 ? (0, _util.getPluralName)(resourceName) : _ref$resourcePluralNa,
      _ref$isArray = _ref.isArray,
      isArray = _ref$isArray === void 0 ? false : _ref$isArray;
  return resourceName ? "".concat(actionId.toUpperCase(), "_").concat((0, _util.upperSnakeCase)(isArray ? resourcePluralName : resourceName)) : (0, _util.upperSnakeCase)(actionId);
};

exports.getActionTypeKey = getActionTypeKey;

var getActionType = function getActionType(actionId) {
  return (0, _util.upperSnakeCase)(actionId);
};

exports.getActionType = getActionType;

var createType = function createType(actionId, _ref2) {
  var resourceName = _ref2.resourceName,
      resourcePluralName = _ref2.resourcePluralName,
      _ref2$isArray = _ref2.isArray,
      isArray = _ref2$isArray === void 0 ? false : _ref2$isArray,
      alias = _ref2.alias;
  var typeKey = getActionTypeKey(resourceName ? alias || actionId : actionId, {
    resourceName: resourceName,
    resourcePluralName: resourcePluralName,
    isArray: isArray
  });
  return (0, _defineProperty2["default"])({}, typeKey, getActionType(actionId));
};

exports.createType = createType;

var createTypes = function createTypes() {
  var actions = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  var _ref4 = arguments.length > 1 ? arguments[1] : undefined,
      resourceName = _ref4.resourceName,
      resourcePluralName = _ref4.resourcePluralName,
      _ref4$scope = _ref4.scope,
      scope = _ref4$scope === void 0 ? getTypesScope(resourceName) : _ref4$scope;

  var rawTypes = Object.keys(actions).reduce(function (types, actionId) {
    var actionOpts = actions[actionId];
    return Object.assign(types, createType(actionId, {
      resourceName: resourceName,
      resourcePluralName: resourcePluralName,
      isArray: actionOpts.isArray,
      alias: actionOpts.alias
    }));
  }, {});
  return scopeTypes(rawTypes, scope);
};

exports.createTypes = createTypes;
//# sourceMappingURL=types.js.map