/// <reference types="src/typings" />
import { Action as ReduxAction, Reducer as ReduxReducer } from 'redux';
import { ThunkAction } from 'redux-thunk';
import { SerializableResponse } from 'src/helpers/fetch';
export declare type UnknownObject = Record<string, unknown>;
export declare type SupportedReduceConfigActionOptionKeys = 'invalidateState' | 'assignResponse' | 'mergeResponse' | 'isArray' | 'isPure';
export declare type SupportedReduceConfigActionOptions = Pick<ReduceOptions, SupportedReduceConfigActionOptionKeys>;
export declare type SupportedFetchConfigActionOptionKeys = 'url' | 'method' | 'headers' | 'credentials' | 'query' | 'body';
export declare type SupportedFetchConfigActionOptions = Pick<FetchOptions, SupportedFetchConfigActionOptionKeys>;
export declare type ConfigActionOptions = SupportedReduceConfigActionOptions & SupportedFetchConfigActionOptions & {
    alias?: string;
    gerundName?: string;
    name?: string;
};
export declare type DefaultActionVerb = 'create' | 'fetch' | 'get' | 'update' | 'updateMany' | 'delete' | 'deleteMany';
export declare type ConfigActionsOptions = Record<DefaultActionVerb | string, ConfigActionOptions>;
export declare type State<T = UnknownObject> = {
    didInvalidate: boolean;
    didInvalidateItem: boolean;
    isCreating: boolean;
    isDeleting: boolean;
    isFetching: boolean;
    isFetchingItem: boolean;
    isUpdating: boolean;
    item: T | null;
    items: Array<T>;
    lastUpdated: number;
    lastUpdatedItem: number;
};
export declare type ReduceOptions<T extends UnknownObject = UnknownObject> = {
    invalidateState?: boolean;
    assignResponse?: boolean;
    mergeResponse?: boolean;
    isArray?: boolean;
    isPure?: boolean;
    reduce?: Reducer<T>;
    mergeItem?: (prev: T | null, next: Partial<T>) => T;
    gerundName?: string;
    params?: Record<string, string>;
};
export declare type FetchOptions = Pick<RequestInit, 'method' | 'headers' | 'credentials' | 'signal'> & {
    url?: string;
    query?: Record<string, unknown>;
    body?: string | Record<string, unknown> | Array<unknown>;
    Promise?: PromiseConstructor;
};
export declare type Context = undefined | string | Record<string, unknown>;
export declare type ContextOptions = Partial<FetchOptions & ReduceOptions>;
export declare type ContentRange = {
    unit: string | number;
    first: number;
    last: number;
    length: number | null;
};
export declare type Types = Record<string, string>;
export declare type Action<T = unknown> = ReduxAction<string> & {
    status: 'pending' | 'resolved' | 'rejected';
    options: ContextOptions;
    context: Context;
    payload?: Partial<SerializableResponse<T>>;
};
export declare type Reducer<T = UnknownObject> = ReduxReducer<State<T>, Action>;
export declare type ReducerMapObject<T = UnknownObject> = Record<string, Reducer<T>>;
export declare type AsyncActionCreator<T = unknown> = (context?: Context, contextOpts?: ContextOptions) => ThunkAction<Promise<unknown>, State, void, Action<T>>;
export declare type AsyncActionCreatorsMapObject<T = unknown> = Record<string, AsyncActionCreator<T>>;
export declare type UnwrapAsyncActionCreatorsMapObject<T extends Record<string, AsyncActionCreator>> = T extends AsyncActionCreatorsMapObject<infer A> ? Record<string, (context?: Context, contextOpts?: ContextOptions) => Promise<Action<A>>> : T;
export declare type BeforeErrorPipeline = Array<(err: Error) => Error | null>;
